const httpServer = require("http").createServer();
const io = require("socket.io")(httpServer, {pingTimeout:10000, pingInterval:5000});

var routes = new Map();

function mapString(m) {
  let tmp = [];
  m.forEach((v, k) => { tmp.push(`[${k}]:${v}`) });
  return `{${tmp.join(',\n')}}`
}

function timeStampString4() {
  const interval = Date.now() % 10000;
  if (interval > 999) {
    return `${interval}`;
  } else if (interval > 99) {
    return `0${interval}`;
  } else if (interval > 9) {
    return `00${interval}`;
  } else {
    return `000${interval}`;
  }
}

io.on("connection", (socket) => {
  console.info(`client connected socket => ${socket.id}`);
  
  socket.on('bind', (bindId, callback) => {
    console.info(`bind peer => [${socket.id} -> ${bindId}]`);

    io.clients((error, clients) => {
      if (error) throw error;
      if (clients.some( s => s === bindId)) {
        routes.set(socket.id, bindId);
        console.info(`bind success`);
        console.debug(`routes => ${mapString(routes)}`);
        callback(true);
      } else {
        console.warn('peer is not connected');
        callback(false);
      }
    });
    
  });

  socket.on('msg', (data) => {

    console.info(`[ts-${timeStampString4()}]: from socket => ${socket.id}`);

    if (!routes.has(socket.id)) {
      console.warn(`peer not find for socket => ${socket.id}`);
      return;
    }

    const bindId = routes.get(socket.id);
    console.info(`to socket => ${bindId}`);

    if (process.env.NODE_ENV !== 'production') {
      console.debug(`message => ${data}`);
    }

    socket.to(bindId).emit('msg',  socket.id, data);
  })

  socket.on('disconnect', (reason)=> {
    console.info(`disconnect socket => ${socket.id} reason => ${reason}`);
    
    const toDelete = [];
    routes.forEach((value, key) => {
      if (value === socket.id || key === socket.id) {
        toDelete.push(key);
      }
    })
    toDelete.forEach(k => routes.delete(k));

    console.debug(`routes => ${mapString(routes)}`);
  })
});

httpServer.listen(3001);